---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons

---------------------------
-- create the button
---------------------------
gf.AddButton("Mounts","RIGHT")
local btn = gb.Mounts
btn.text              = "      "
btn.enabled           = true
btn.texture.normal    = "Interface\\AddOns\\Glance_Mounts\\mount.tga"
btn.events            = {"COMPANION_LEARNED"}
btn.update            = true
btn.click             = true
btn.tooltip           = true
btn.menu              = true
btn.options           = true

---------------------------
-- shortcuts
---------------------------
local spa = btn.save.perAccount
local HEX = ga.colors.HEX
local tooltip = gf.Tooltip

---------------------------
-- not available till lvl 30
---------------------------
if UnitLevel("player") < 20 then btn.enabled = false end

---------------------------
-- commands
---------------------------
gf.AddCommand("Mounts","random","Summon a random mount",function() if IsMounted() then C_MountJournal.Dismiss() else gf.Mounts.summonRandom() end end)
gf.AddCommand("Mounts","random-air","Summon a random flying mount",function() if IsMounted() then C_MountJournal.Dismiss() else gf.Mounts.summonRandomAir() end end)
gf.AddCommand("Mounts","random-land","Summon a random ground mount",function() if IsMounted() then C_MountJournal.Dismiss() else gf.Mounts.summonRandomLand() end end)
gf.AddCommand("Mounts","random-water","Summon a random water mount",function() if IsMounted() then C_MountJournal.Dismiss() else gf.Mounts.summonRandomWater() end end)

---------------------------
-- variables
---------------------------
ga.Flags = {
	--land, air, sea, swim, water walk, location lock, dead
	[230] = {true, false, false, true, false, false, false}, -- ground mounts
	[231] = {true, false, true, true, false, false, false}, -- riding turtle and sea turtle
	[232] = {false, false, true, true, false, true, false}, -- vash seahorses
	[241] = {true, false, false, false, false, true, false}, -- battle tanks
	[242] = {false, true, false, false, false, false, true}, -- spectral gryphon
	[247] = {true, true, false, false, false, false, false}, -- red flying cloud no swimming
	[248] = {true, true, false, true, false, false, false}, -- flying mounts
	[254] = {false, false, true, true, false, false, false}, -- subdued seahorse
	[269] = {true, false, false, true, true, false, false}, -- water striders
}	

ga.alphabet = {
	["a"] = "ae",
	["b"] = "ae",
	["c"] = "ae",
	["d"] = "ae",
	["e"] = "ae",
	["f"] = "fj",
	["g"] = "fj",
	["h"] = "fj",
	["i"] = "fj",
	["j"] = "fj",
	["k"] = "ko",
	["l"] = "ko",
	["m"] = "ko",
	["n"] = "ko",
	["o"] = "ko",
	["p"] = "pt",
	["q"] = "pt",
	["r"] = "pt",
	["s"] = "pt",
	["t"] = "pt",
	["u"] = "uz",
	["v"] = "uz",
	["w"] = "uz",
	["x"] = "uz",
	["y"] = "uz",
	["z"] = "uz",
}
ga.tmpmount = {}

---------------------------
-- update
---------------------------
function gf.Mounts.update(self, event, arg1)
	if event == "COMPANION_LEARNED" then
		gf.createMenu("Mounts")
	end
end

---------------------------
-- tooltip
---------------------------
function gf.Mounts.tooltip()
	Glance.Debug("function","tooltip","Mounts")
	tooltip.Title("Mounts", "GLD")
	tooltip.Line("You have collected "..HEX.green..gf.Mounts.GetNumMounts()..HEX.white.." mounts.","WHT")
	tooltip.Space()
	tooltip.Line("Legend", "GLD")
	tooltip.Line(HEX.lightblue.."Blue |rmounts are favorites.","WHT")
	tooltip.Line(HEX.red.."Red |rmounts cannot be used in this area.","WHT")
	tooltip.Notes("summon or dismiss a random mount",nil,"summon a specific mount",nil,"Only favorite mounts will be summoned randomly")
end

---------------------------
-- click
---------------------------
function gf.Mounts.click(self, button, down)
	Glance.Debug("function","click","Mounts")
	if button == "LeftButton" then
		if IsMounted() then
			C_MountJournal.Dismiss()
		else
			C_MountJournal.Summon(0)
		end
	end
end

---------------------------
-- menu
---------------------------
function gf.Mounts.menu(level,UIDROPDOWNMENU_MENU_VALUE)
	Glance.Debug("function","menu","Mounts")
	level = level or 1
	if (level == 1) then
		gf.setMenuTitle("Mounts Options")
		gf.setMenuHeader("Flying","flying",level)
		gf.setMenuHeader("Riding","riding",level)
		gf.setMenuHeader("Swimming","swimming",level)
		gf.setMenuHeader("Water Walking","walking",level)
	end
	if (level == 2) then
		if gf.isMenuValue("flying") then
			gf.setMenuTitle("     Flying Mounts",level, true)
			gf.Mounts.sortMenuLines("f",level)
		end
		if gf.isMenuValue("riding") then
			gf.setMenuTitle("     Riding Mounts",level, true)
			gf.Mounts.sortMenuLines("r",level)
		end
		if gf.isMenuValue("swimming") then
			gf.setMenuTitle("     Swimming Mounts",level, true)
			for i=1,C_MountJournal.GetNumMounts() do
				gf.Mounts.writeMenuLine(level,i,"s")
			end
		end
		if gf.isMenuValue("walking") then
			gf.setMenuTitle("     Water Walking Mounts",level, true)
			for i=1,C_MountJournal.GetNumMounts() do
				gf.Mounts.writeMenuLine(level,i,"w")
			end
		end
	end
	if (level == 3) then
		for i=1,C_MountJournal.GetNumMounts() do
			local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar = gf.Mounts.mountInfo(i)
			if air and string.sub(UIDROPDOWNMENU_MENU_VALUE,1,1) == "f" then
				if ga.alphabet[strlower(string.sub(creatureName,1,1))] == string.sub(UIDROPDOWNMENU_MENU_VALUE,2,3) then -- alpha table from name matches ui dropdown 2nd-3rd letter
					gf.Mounts.writeMenuLine(level,i,UIDROPDOWNMENU_MENU_VALUE)
				end
			end
			if land and string.sub(UIDROPDOWNMENU_MENU_VALUE,1,1) == "r" then
				if ga.alphabet[strlower(string.sub(creatureName,1,1))] == string.sub(UIDROPDOWNMENU_MENU_VALUE,2,3) then -- alpha table from name matches ui dropdown 2nd-3rd letter
					gf.Mounts.writeMenuLine(level,i,UIDROPDOWNMENU_MENU_VALUE)
				end
			end
		end
	end
end

---------------------------
-- sort menu line
---------------------------
function gf.Mounts.sortMenuLines(which,level)
	gf.setMenuHeader("Mounts A-E",which.."ae",level)
	gf.setMenuHeader("Mounts F-J",which.."fj",level)
	gf.setMenuHeader("Mounts K-O",which.."ko",level)
	gf.setMenuHeader("Mounts P-T",which.."pt",level)
	gf.setMenuHeader("Mounts U-Z",which.."uz",level)
end

---------------------------
-- write menu line
---------------------------
function gf.Mounts.writeMenuLine(level,i,which)
	local HEX = ga.colors.HEX; local color = HEX.white;
	local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar, mountFlags = gf.Mounts.mountInfo(i)
	if not isCollected then return end
	if which == "f" and not air then return end -- looking for air
	if which == "r" and ((not land) or (sea or air)) then return end -- looking for land
	if which == "s" and ((not sea) or (air)) then return end -- looking for sea
	if which == "w" and ((not waterWalk) or air) then return end -- looking for waterWalk
	local checked, func
	checked = gf.Mounts.isSummoned(i)
	func = function() gf.Mounts.summon(i) end;
	if not gf.Mounts.isUsable(i) then
		color = HEX.red
	elseif isFavorite then 
		color = HEX.lightblue -- color blue others that are favorites and can be summoned
	end
	--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func,icon,keepShown,notRadio
	gf.setInfo(nil,false,false,false,checked,color..creatureName,name,level,func,icon,false,false)
end

---------------------------
-- options
---------------------------
function gf.Mounts.options()
	local pIDX = gf.createPanel("Macros","The following macros can be made to randomize your mounts.  The choice of mount is determined by your settings in the Mounts panel.",nil)
	gf.createText(pIDX,15,-60,"/glance random","GameFontNormal","")
	gf.createText(pIDX,15,-75,"Summons a random mount best suited for the area you are in.","GameFontHighlight","")
	gf.createText(pIDX,15,-110,"/glance randomair","GameFontNormal","")
	gf.createText(pIDX,15,-125,"Summons a random flying mount.","GameFontHighlight","")
	gf.createText(pIDX,15,-160,"/glance randomland","GameFontNormal","")
	gf.createText(pIDX,15,-175,"Summons a random land mount.","GameFontHighlight","")
	gf.createText(pIDX,15,-210,"/glance randomwater","GameFontNormal","")
	gf.createText(pIDX,15,-225,"Summons a random water mount.","GameFontHighlight","")
end

---------------------------
-- actual mount count
---------------------------
function gf.Mounts.GetNumMounts()
	local count = 0
	for i=1, C_MountJournal.GetNumMounts() do
		if select(11,C_MountJournal.GetMountInfo(i)) then
			count = count + 1
		end
	end
	return count
end

---------------------------
-- can use mount here?
---------------------------
function gf.Mounts.isUsable(id)
	return select(5,C_MountJournal.GetMountInfo(id))
end

---------------------------
-- is summoned
---------------------------
function gf.Mounts.isSummoned(id)
	return select(4,C_MountJournal.GetMountInfo(id))
end

---------------------------
-- summon
---------------------------
function gf.Mounts.summon(id)
	if select(4,C_MountJournal.GetMountInfo(id)) then
		C_MountJournal.Dismiss(id)
	else
		C_MountJournal.Summon(id)
	end
end

---------------------------
-- get mount info
---------------------------
function gf.Mounts.mountInfo(id)
	local creatureName, creatureSpellID, icon, issummoned, isUsable, sourceType, isFavorite, isFactionSpecific, faction, hideOnChar, isCollected = C_MountJournal.GetMountInfo(id)
	local creatureDisplayID, descriptionText, sourceText, isSelfMount, mountFlags = C_MountJournal.GetMountInfoExtra(id)
	if (mountFlags ~= nil and ga.Flags[mountFlags] ~= nil) then
		local land, air, sea, swim, waterWalk, locationLock, dead = unpack(ga.Flags[mountFlags])
		return creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar, mountFlags
	end
end

---------------------------
-- no fly zones
---------------------------
function gf.Mounts.NoFlyZone()
	if not IsFlyableArea() then -- wow says no
		return true;
	end
	local pvpID, localizedName, isActive, canQueue, sec, canEnter =  (1)
	if (isActive and GetZoneText()=="Wintergrasp") then -- in battleground, wow should have said no.
		return true;
	end
	pvpID, localizedName, isActive, canQueue, sec, canEnter = GetWorldPVPAreaInfo(2)
	if (GetZoneText()=="Tol Barad") then -- in battleground, wow should have said no. isActive doesn't seem to matter here
		return true;
	end
	if GetZoneText()=="Isle of Giants" or GetZoneText()=="Isle of Thunder" or GetZoneText()=="Timeless Isle" then -- evil blizzard
		return true
	end
	return false;
end

---------------------------
-- summon a random mount
---------------------------
function gf.Mounts.summonRandom()
	Glance.Debug("function","summon","Random")
	if IsOutdoors() then
		if IsSwimming() then
			gf.Mounts.summonRandomWater()
		else
			if gf.Mounts.NoFlyZone() then
				gf.Mounts.summonRandomLand()
			else
				gf.Mounts.summonRandomAir()
			end
		end
	end
	--C_MountJournal.Summon(0)
end

---------------------------
-- summon a random air mount
---------------------------
function gf.Mounts.summonRandomAir()
	local mountCount = 0
	for i=1,C_MountJournal.GetNumMounts() do
		local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar = gf.Mounts.mountInfo(i)
		if air and isCollected and isFavorite and gf.Mounts.isUsable(i) then
			mountCount = mountCount + 1
			ga.tmpmount[mountCount] = i
		end
	end
	gf.Mounts.Randomize("Air")
end

---------------------------
-- summon a random land mount
---------------------------
function gf.Mounts.summonRandomLand()
	local mountCount = 0
	if GetZoneText() == "Ahn'Qiraj" then -- in the temple, launch only battle tanks
		for i=1,C_MountJournal.GetNumMounts() do
			local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar = gf.Mounts.mountInfo(i)
			if land and isCollected and not air then
				if strfind(creatureName,"Qiraji") then -- allow the battle tank
					mountCount = mountCount + 1
					ga.tmpmount[mountCount] = i
				end
			end
		end
	elseif not GetZoneText() == "Ahn'Qiraj" or mountCount == 0 then -- not in the temple or no battle tanks found
		for i=1,C_MountJournal.GetNumMounts() do
			local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar = gf.Mounts.mountInfo(i)
			if land and isCollected and isFavorite and gf.Mounts.isUsable(i) and not air and (not strfind(creatureName,"Qiraji")) then
				mountCount = mountCount + 1
				ga.tmpmount[mountCount] = i
			end
		end
	end
	gf.Mounts.Randomize("Land")
end

---------------------------
-- summon a random water mount
---------------------------
function gf.Mounts.summonRandomWater()
	local mountCount = 0
	for i=1,C_MountJournal.GetNumMounts() do
		local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar = gf.Mounts.mountInfo(i)
		if sea and isCollected and not land and not air then
			if strfind(creatureName,"Vash") then -- allow the seahorse
				if strfind(GetMapInfo(),"Vash") then
					mountCount = mountCount + 1
					ga.tmpmount[mountCount] = i
					gf.Mounts.summon(i) -- can only be used here, just call it
					return
				end
			else
				mountCount = mountCount + 1
				ga.tmpmount[mountCount] = i
			end
		end
	end
	if mountCount==0 then -- no sea mounts available, use one that can swim
		for i=1,C_MountJournal.GetNumMounts() do
			local creatureName, icon, issummoned, land, air, sea, swim, waterWalk, isFavorite, isCollected, isSelfMount, hideOnChar = gf.Mounts.mountInfo(i)
			if swim and isCollected and isFavorite and gf.Mounts.isUsable(i) then
				if strfind(creatureName,"Strider") then -- favor the water strider
					mountCount = mountCount + 1
					ga.tmpmount[mountCount] = i
					gf.Mounts.summon(i) -- just call it
					return
				end
				mountCount = mountCount + 1
				ga.tmpmount[mountCount] = i
			end
		end
	end
	gf.Mounts.Randomize("Water")
end

---------------------------
-- randomize
---------------------------
function gf.Mounts.Randomize(which)
	if #ga.tmpmount > 0 then
		local rdm = ga.tmpmount[random(1,#ga.tmpmount)]
		gf.Mounts.summon(rdm)
		Glance.Debug("function","summon","Random")
	end
	wipe(ga.tmpmount)
end

---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Mounts")
end